@@ -2,10 +2,11 @@ package Tree::DAG_Node;
 
 use strict;
 use warnings;
-use warnings  qw(FATAL utf8); # Fatalize encoding glitches.
+use warnings qw(FATAL utf8); # Fatalize encoding glitches.
+use open     qw(:std :utf8); # Undeclared streams in UTF-8.
 
 our $Debug   = 0;
-our $VERSION = '1.24';
+our $VERSION = '1.25';
 
 use File::Slurp::Tiny 'read_lines';
 
@@ -372,6 +373,45 @@ sub daughters { # read-only attrib-method: returns a list.
   return @{$this->{'daughters'} || []};
 }
 
+# ------------------------------------------------
+
+sub decode_lol
+{
+	my($self, $result) = @_;
+	my(@worklist)      = $result;
+
+	my($obj);
+	my($ref_type);
+	my(@stack);
+
+	do
+	{
+		$obj      = shift @worklist;
+		$ref_type = ref $obj;
+
+		if ($ref_type eq 'ARRAY')
+		{
+			unshift @worklist, @$obj;
+		}
+		elsif ($ref_type eq 'HASH')
+		{
+			push @stack, {%$obj};
+		}
+		elsif ($ref_type)
+		{
+			die "Unsupported object type $ref_type\n";
+		}
+		else
+		{
+			push @stack, $obj;
+		}
+
+	} while (@worklist);
+
+	return [@stack];
+
+} # End of decode_lol.
+
 # -----------------------------------------------
 
 sub delete_tree {
@@ -632,7 +672,7 @@ sub dump_names {
         join('',
              $o->{'indent'} x $o->{'_depth'},
              $o->{'tick'},
-             &Tree::DAG_Node::_dump_quote(defined $this->name ? $this->name : $this),
+             defined $this->name ? $this->name : $this,
              "\n"
         )
       );
@@ -645,32 +685,6 @@ sub dump_names {
 
 # -----------------------------------------------
 
-sub _dump_quote {
-  my @stuff = @_;
-  return
-    join(", ",
-    map
-     { # the cleaner-upper function
-       if(!length($_)) { # empty string
-         "''";
-       } elsif( m/^-?\d+(?:\.\d+)?$/s ) { # a number
-         $_;
-       } elsif( # text with junk in it
-          s<([^\x20\x21\x23\x27-\x3F\x41-\x5B\x5D-\x7E])>
-           <'\\x'.(unpack("H2",$1))>eg
-         ) {
-         "\"$_\"";
-       } else { # text with no junk in it
-         s<'><\\'>g;
-         "\'$_\'";
-       }
-     }
-     @stuff
-    );
-}
-
-# -----------------------------------------------
-
 sub format_node
 {
 	my($self, $options, $node) = @_;
@@ -995,6 +1009,16 @@ sub node2string
 
 # -----------------------------------------------
 
+sub quote_name
+{
+	my($self, $name) = @_;
+
+	return "'$name'";
+
+} # End of quote_name.
+
+# -----------------------------------------------
+
 sub random_network { # constructor or method.
   my $class = $_[0];
   my $o = ref($_[1]) ? $_[1] : {};
@@ -1388,6 +1412,80 @@ sub string2hashref
 
 # -----------------------------------------------
 
+sub tree_to_lol {
+  # I haven't /rigorously/ tested this.
+  my($it, $o) = @_[0,1]; # $o is currently unused anyway
+  $o = {} unless ref $o;
+
+  my $out = [];
+  my @lol_stack = ($out);
+  $o->{'callback'} = sub {
+      my($this, $o) = @_[0,1];
+      my $new = [];
+      push @{$lol_stack[-1]}, $new;
+      push(@lol_stack, $new);
+      return 1;
+    }
+  ;
+  $o->{'callbackback'} = sub {
+      my($this, $o) = @_[0,1];
+      my $name = defined $this->name ? $it -> quote_name($this->name) : 'undef';
+      push @{$lol_stack[-1]}, $name;
+      pop @lol_stack;
+      return 1;
+    }
+  ;
+  $it->walk_down($o);
+  die "totally bizarre error 12416" unless ref($out->[0]);
+  $out = $out->[0]; # the real root
+  return $out;
+}
+
+# -----------------------------------------------
+
+sub tree_to_lol_notation {
+  my($it, $o) = @_[0,1];
+  $o = {} unless ref $o;
+  my @out = ();
+  $o->{'_depth'} ||= 0;
+  $o->{'multiline'} = 0 unless exists($o->{'multiline'});
+
+  my $line_end;
+  if($o->{'multiline'}) {
+    $o->{'indent'} ||= '  ';
+    $line_end = "\n";
+  } else {
+    $o->{'indent'} ||= '';
+    $line_end = '';
+  }
+
+  $o->{'callback'} = sub {
+      my($this, $o) = @_[0,1];
+      push(@out,
+             $o->{'indent'} x $o->{'_depth'},
+             "[$line_end",
+      );
+      return 1;
+    }
+  ;
+  $o->{'callbackback'} = sub {
+      my($this, $o) = @_[0,1];
+      my $name = defined $this->name ? $it -> quote_name($this->name) : 'undef';
+      push(@out,
+             $o->{'indent'} x ($o->{'_depth'} + 1),
+             "$name$line_end",
+             $o->{'indent'} x $o->{'_depth'},
+             "],$line_end",
+      );
+      return 1;
+    }
+  ;
+  $it->walk_down($o);
+  return join('', @out);
+}
+
+# -----------------------------------------------
+
 sub tree_to_simple_lol {
   # I haven't /rigorously/ tested this.
   my $root = $_[0];
@@ -1403,7 +1501,8 @@ sub tree_to_simple_lol {
   $o->{'callback'} = sub {
       my($this, $o) = @_[0,1];
       my $new;
-      $new = scalar($this->daughters) ? [] : $this->name;
+      my $name = defined $this->name ? $it -> quote_name($this->name) : 'undef';
+      $new = scalar($this->daughters) ? [] : $name;
         # Terminal nodes are scalars, the rest are listrefs we'll fill in
         # as we recurse the tree below here.
       push @{$lol_stack[-1]}, $new;
@@ -1444,11 +1543,10 @@ sub tree_to_simple_lol_notation {
                "[$line_end",
         );
       } else {   # Terminal
-        my $name = $this->name;
+        my $name = defined $this->name ? $it -> quote_name($this->name) : 'undef';
         push @out,
           $o->{'indent'} x $o->{'_depth'},
-          defined($name) ? &Tree::DAG_Node::_dump_quote($name) : 'undef',
-          ",$line_end";
+          "$name,$line_end";
       }
       return 1;
     }
@@ -1469,80 +1567,6 @@ sub tree_to_simple_lol_notation {
 
 # -----------------------------------------------
 
-sub tree_to_lol {
-  # I haven't /rigorously/ tested this.
-  my($it, $o) = @_[0,1]; # $o is currently unused anyway
-  $o = {} unless ref $o;
-
-  my $out = [];
-  my @lol_stack = ($out);
-  $o->{'callback'} = sub {
-      my($this, $o) = @_[0,1];
-      my $new = [];
-      push @{$lol_stack[-1]}, $new;
-      push(@lol_stack, $new);
-      return 1;
-    }
-  ;
-  $o->{'callbackback'} = sub {
-      my($this, $o) = @_[0,1];
-      push @{$lol_stack[-1]}, $this->name;
-      pop @lol_stack;
-      return 1;
-    }
-  ;
-  $it->walk_down($o);
-  die "totally bizarre error 12416" unless ref($out->[0]);
-  $out = $out->[0]; # the real root
-  return $out;
-}
-
-# -----------------------------------------------
-
-sub tree_to_lol_notation {
-  my $root = $_[0];
-  my($it, $o) = @_[0,1];
-  $o = {} unless ref $o;
-  my @out = ();
-  $o->{'_depth'} ||= 0;
-  $o->{'multiline'} = 0 unless exists($o->{'multiline'});
-
-  my $line_end;
-  if($o->{'multiline'}) {
-    $o->{'indent'} ||= '  ';
-    $line_end = "\n";
-  } else {
-    $o->{'indent'} ||= '';
-    $line_end = '';
-  }
-
-  $o->{'callback'} = sub {
-      my($this, $o) = @_[0,1];
-      push(@out,
-             $o->{'indent'} x $o->{'_depth'},
-             "[$line_end",
-      );
-      return 1;
-    }
-  ;
-  $o->{'callbackback'} = sub {
-      my($this, $o) = @_[0,1];
-      my $name = defined $this->name ? &Tree::DAG_Node::_dump_quote($this->name) : 'undef';
-      push(@out,
-             $o->{'indent'} x ($o->{'_depth'} + 1),
-             "$name$line_end",
-             $o->{'indent'} x $o->{'_depth'},
-             "], $line_end",
-      );
-      return 1;
-    }
-  ;
-  $it->walk_down($o);
-  return join('', @out);
-}
-
-# -----------------------------------------------
-
 sub tree2string
 {
 	my($self, $options, $tree) = @_;
@@ -1666,6 +1690,8 @@ sub walk_down {
 
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 Tree::DAG_Node - An N-ary tree
@@ -1680,7 +1706,7 @@ Tree::DAG_Node - An N-ary tree
 
 	# Now add your own methods overriding/extending the methods in C<Tree::DAG_Node>...
 
-=head2 Using as a class of its own
+=head2 Using as a class on its own
 
 	use Tree::DAG_Node;
 
@@ -1705,11 +1731,13 @@ Or:
 
 =head2 Using with utf-8 data
 
-	read_tree($file_name) works with utf-8 data. See t/read.tree.t and t/tree.utf8.attributes.txt.
-	Such a file can be created by redirecting the output of tree2string() to a file of type utf-8.
+read_tree($file_name) works with utf-8 data. See t/read.tree.t and t/tree.utf8.attributes.txt.
+Such a file can be created by redirecting the output of tree2string() to a file of type utf-8.
 
 See the docs for L<Encode> for the difference between utf8 and utf-8. In brief, use utf-8.
 
+See also scripts/write_tree.pl and scripts/read.tree.pl and scripts/read.tree.log.
+
 =head1 DESCRIPTION
 
 This class encapsulates/makes/manipulates objects that represent nodes
@@ -2143,6 +2171,14 @@ Options you specify are passed down to calls to $node->copy.
 
 This returns the (possibly empty) list of daughters for $node.
 
+=head2 decode_lol($lol)
+
+Returns an arrayref having decoded the deeply nested structure $lol.
+
+$lol will be the output of either tree_to_lol() or tree_to_simple_lol().
+
+See scripts/read.tree.pl, and it's output file scripts/read.tree.log.
+
 =head2 delete_tree()
 
 Destroys the entire tree that $node is a member of (starting at the
@@ -2325,8 +2361,6 @@ Default: ' ' x 2.
 The output is not printed, but is returned as a list, where each
 item is a line, with a "\n" at the end.
 
-Note: Names are converted to a printable form using the undocumented function _dump_quote().
-
 =head2 format_node($options, $node)
 
 Returns a string consisting of the node's name and, optionally, it's attributes.
@@ -2592,6 +2626,10 @@ Calls L</format_node($options, $node)>.
 
 Called by L</tree2string($options, [$some_tree])>.
 
+=head2 quote_name($name)
+
+Returns the string "'$name'", which is used in various methods for outputting node names.
+
 =head2 random_network($options)
 
 This method can be called as a class method or as an object method.
@@ -2843,6 +2881,10 @@ $lol, above, holds.  (This is as opposed to L</tree_to_lol_notation($options)>,
 which returns the viewable code like what gets evaluated and stored in
 $lol, above.)
 
+Undefined node names are returned as the string 'undef'.
+
+See also L</decode_lol($lol)>.
+
 Lord only knows what you use this for -- maybe for feeding to
 Data::Dumper, in case L</tree_to_lol_notation($options)> doesn't do just what you
 want?
@@ -2874,7 +2916,7 @@ Doing this:
 prints the same content, just spread over many lines, and prettily
 indented.
 
-Note: Names are converted to a printable form using the undocumented function _dump_quote().
+Undefined node names are returned as the string 'undef'.
 
 =head2 tree_to_simple_lol()
 
@@ -2888,12 +2930,16 @@ the same as $node->name.
 
 Compare to tree_to_simple_lol_notation.
 
+Undefined node names are returned as the string 'undef'.
+
+See also L</decode_lol($lol)>.
+
 =head2 tree_to_simple_lol_notation($options)
 
 A simple-LoL version of tree_to_lol_notation (which see); takes the
 same options.
 
-Note: Names are converted to a printable form using the undocumented function _dump_quote().
+Undefined node names are returned as the string 'undef'.
 
 =head2 tree2string($options, [$some_tree])
 
@@ -2905,59 +2951,53 @@ Draws a nice ASCII-art representation of the tree structure.
 
 The tree looks like:
 
-	Root. Attributes: {# => "0"}
-	   |--- I. Attributes: {# => "1"}
-	   |   |--- J. Attributes: {# => "3"}
-	   |   |   |--- K. Attributes: {# => "3"}
-	   |   |--- J. Attributes: {# => "4"}
-	   |       |--- L. Attributes: {# => "5"}
-	   |           |--- M. Attributes: {# => "5"}
-	   |               |--- N. Attributes: {# => "5"}
-	   |                   |--- O. Attributes: {# => "5"}
-	   |--- H. Attributes: {# => "2"}
-	   |   |--- J. Attributes: {# => "3"}
-	   |   |   |--- K. Attributes: {# => "3"}
-	   |   |--- J. Attributes: {# => "4"}
-	   |       |--- L. Attributes: {# => "5"}
-	   |           |--- M. Attributes: {# => "5"}
-	   |               |--- N. Attributes: {# => "5"}
-	   |                   |--- O. Attributes: {# => "5"}
-	   |--- D. Attributes: {# => "6"}
-	   |   |--- F. Attributes: {# => "8"}
-	   |       |--- G. Attributes: {# => "8"}
-	   |--- E. Attributes: {# => "7"}
-	   |   |--- F. Attributes: {# => "8"}
-	   |       |--- G. Attributes: {# => "8"}
-	   |--- B. Attributes: {# => "9"}
-	       |--- C. Attributes: {# => "9"}
+	Root. Attributes: {}
+	    |--- Â. Attributes: {# => "ÂÂ"}
+	    |    |--- â. Attributes: {# => "ââ"}
+	    |    |    |--- É. Attributes: {# => "ÉÉ"}
+	    |    |--- ä. Attributes: {# => "ää"}
+	    |    |--- é. Attributes: {# => "éé"}
+	    |         |--- Ñ. Attributes: {# => "ÑÑ"}
+	    |              |--- ñ. Attributes: {# => "ññ"}
+	    |                   |--- Ô. Attributes: {# => "ÔÔ"}
+	    |                        |--- ô. Attributes: {# => "ôô"}
+	    |                        |--- ô. Attributes: {# => "ôô"}
+	    |--- ß. Attributes: {# => "ßß"}
+	         |--- ®. Attributes: {# => "®®"}
+	         |    |--- ©. Attributes: {# => "©©"}
+	         |--- £. Attributes: {# => "££"}
+	         |--- €. Attributes: {# => "€€"}
+	         |--- √. Attributes: {# => "√√"}
+	         |--- ×xX. Attributes: {# => "×xX×xX"}
+	              |--- í. Attributes: {# => "íí"}
+	              |--- ú. Attributes: {# => "úú"}
+	              |--- «. Attributes: {# => "««"}
+	              |--- ». Attributes: {# => "»»"}
 
 Or, without attributes:
 
 	Root
-	   |--- I
-	   |   |--- J
-	   |   |   |--- K
-	   |   |--- J
-	   |       |--- L
-	   |           |--- M
-	   |               |--- N
-	   |                   |--- O
-	   |--- H
-	   |   |--- J
-	   |   |   |--- K
-	   |   |--- J
-	   |       |--- L
-	   |           |--- M
-	   |               |--- N
-	   |                   |--- O
-	   |--- D
-	   |   |--- F
-	   |       |--- G
-	   |--- E
-	   |   |--- F
-	   |       |--- G
-	   |--- B
-	       |--- C
+	    |--- Â
+	    |    |--- â
+	    |    |    |--- É
+	    |    |--- ä
+	    |    |--- é
+	    |         |--- Ñ
+	    |              |--- ñ
+	    |                   |--- Ô
+	    |                        |--- ô
+	    |                        |--- ô
+	    |--- ß
+	         |--- ®
+	         |    |--- ©
+	         |--- £
+	         |--- €
+	         |--- √
+	         |--- ×xX
+	              |--- í
+	              |--- ú
+	              |--- «
+	              |--- »
 
 See scripts/cut.and.paste.subtrees.pl.
 
